> module Main(main) where
> import Fractals
> import Regions
> import Graphics.Gloss
> import Data.Word
> import qualified Data.ByteString as B

Drawing Fractals with the Gloss library, which drives OpenGL.
(Modified by Andrew from Mark's Hugs graphics library version.)

Do "cabal install gloss" to download and install the gloss package
and its dependencies first.

If you run this under ghci, use the flag -fno-ghci-sandbox 

For cleanliness, we choose to represent our palette using Gloss colors, even though
this can be a bit silly when we are manufacturing the colors from RGBA bytes
(since we have to convert back to bytes to build the bit map!)

This version matches the ppm graphics shown in lecture.

> glossPalette  :: [Color]
> glossPalette   = [ makeColorI ((16*i) `mod` 256)  (8*i)  (255 - 8*i)  255 | i <- [0..31]]  

This alternative version matches the pictures in the JFP paper produced for the Hugs graphics library

> -- glossPalette = cols++cols
> -- col          = [black, cyan, blue, green, magenta, yellow, red, white ]

We need a utility function to get RGBA bytes from Gloss colors.

> rgbaOfColorBS  :: Color -> B.ByteString
> rgbaOfColorBS c = B.pack [floor (r*255),floor (g*255),floor (b*255),floor (a*255)]
>     where (r,g,b,a) = rgbaOfColor c

The renderer generates a bitmap from the grid.

> glossRender    :: String -> Grid Color -> IO ()
> glossRender s g = display (InWindow (s ++  " (press Esc to terminate)") (width,height) (20,20)) white picture
>                where width = length (head g)
>                      height = length g
>                      picture = bitmapOfByteString width height (BitmapFormat TopToBottom PxRGBA) bytes False
>                      bytes = B.concat (map rgbaOfColorBS (concat g))

> main = draw (region1 1024 768) mandelbrot glossPalette (glossRender "mand1")

